/*
 * jasq JavaScript Library Version 2.0
 * http://lmadhavan.com/software/jasq/
 *
 * Copyright (C) 2011, L. Madhavan
 * This software is licensed under the MIT License.
 * The terms of the license can be found at:
 * http://lmadhavan.com/software/jasq/LICENSE.txt
 */

(function () {

var ns = {};

function Class(constructor, methods, staticMethods, superclass) {
	if (superclass)
		constructor.prototype = new superclass();

	if (methods) {
		for (var method in methods)
			if (methods.hasOwnProperty(method))
				constructor.prototype[method] = methods[method];

		if (methods.hasOwnProperty('toString'))
			constructor.prototype.toString = methods.toString;
	}

	if (staticMethods) {
		for (var method in staticMethods)
			if (staticMethods.hasOwnProperty(method))
				constructor[method] = staticMethods[method];
	}

	return constructor;
}

ns.Character = Class(
	function() {
	},
	null,
	{
		forDigit: function(digit, radix) {
			if (radix >= this.MIN_RADIX && radix <= this.MAX_RADIX && digit >= 0 && digit < radix) {
				if (digit < 10)
					return String.fromCharCode('0'.charCodeAt(0) + digit);
				else
					return String.fromCharCode('a'.charCodeAt(0) + digit - 10);
			} else
				return null;
		},

		getNumericValue: function(ch) {
			if (/^[0-9]$/.test(ch))
				return ch.charCodeAt(0) - '0'.charCodeAt(0);
			else if (/^[A-Z]$/.test(ch))
				return ch.charCodeAt(0) - 'A'.charCodeAt(0);
			else if (/^[a-z]$/.test(ch))
				return ch.charCodeAt(0) - 'a'.charCodeAt(0);
			else
				return -1;
		},

		isDigit: function(ch) {
			return /^[0-9\u0660-\u0669\u06F0-\u06F9\u0966-\u096F\uFF10-\uFF19]$/.test(ch);
		},

		isLetter: function(ch) {
			return ch.toLowerCase() != ch.toUpperCase();
		},

		isLetterOrDigit: function(ch) {
			return this.isLetter(ch) || this.isDigit(ch);
		},

		isLowerCase: function(ch) {
			return ch == ch.toLowerCase() && ch != ch.toUpperCase();
		},

		isUpperCase: function(ch) {
			return ch == ch.toUpperCase() && ch != ch.toLowerCase();
		},

		isWhitespace: function(ch) {
			return /^[ \u0009-\u000D\u2000-\u2006\u2008-\u200B\u205F\u3000]$/.test(ch);
		}
	}
);

ns.Formatter = Class(
	function () {
		this.buffer = new ns.StringBuffer();
	},
	{
		format: function(format) {
			for (var i = 0, j = 1; i < format.length; i++)
				if (format.charAt(i) == '%')
					switch (format.charAt(++i)) {
						case 'b': this.buffer.append(new Boolean(arguments[j++])); break;
						case 's': this.buffer.append(arguments[j++]); break;
						case 'd': this.buffer.append(arguments[j++] | 0); break;
						case 'o': this.buffer.append((arguments[j++] | 0).toString(8)); break;
						case 'x': this.buffer.append((arguments[j++] | 0).toString(16)); break;
						case 'X': this.buffer.append((arguments[j++] | 0).toString(16).toUpperCase()); break;
						case 'f': this.buffer.append(arguments[j++] + 0); break;
						case '%': this.buffer.append('%'); break;
						default : throw new Error('Invalid format specifier ' + format.charAt(i-1));
					}
				else
					this.buffer.append(format.charAt(i));
			return this;
		},

		locale: function() {
			return this.locale;
		},

		out: function() {
			return this.buffer;
		},

		toString: function() {
			return this.buffer.toString();
		}
	}
);

ns.Logger = Class(
	function (name, target) {
		this.name = name;
		this.level = ns.Logger.INFO;

		if (!target) target = 'console';
		switch (target) {
			case 'alert': this._log = function(msg) { alert(msg); }; break;
			case 'console': this._log = function(msg) { console.log(msg); }; break;
			case 'document': this._log = function(msg) { document.writeln(msg + '<br/>'); }; break;
			default: throw new Error('Invalid logger target');
		}
	},
	{
		getLevel: function() {
			return this.level;
		},

		setLevel: function(level) {
			this.level = level;
		},

		log: function(level, msg) {
			if (level >= this.level)
				this._log(new Date().toLocaleString() + ' ' + this.name + ' ' + msg);
		},

		config: function(msg) {
			this.log(ns.Logger.CONFIG, 'CONFIG: ' + msg);
		},

		entering: function(method) {
			this.log(ns.Logger.FINER, 'ENTERING: ' + method);
		},

		exiting: function(method) {
			this.log(ns.Logger.FINER, 'EXITING: ' + method);
		},

		info: function(msg) {
			this.log(ns.Logger.INFO, 'INFO: ' + msg);
		},

		severe: function(msg) {
			this.log(ns.Logger.SEVERE, 'SEVERE: ' + msg);
		},

		warning: function(msg) {
			this.log(ns.Logger.WARNING, 'WARNING: ' + msg);
		}
	},
	{
		OFF: 0xFFFFFFFF,
		SEVERE: 1000,
		WARNING: 900,
		INFO: 800,
		CONFIG: 700,
		FINE: 500,
		FINER: 400,
		FINEST: 300,
		ALL: 0
	}
);

ns.Map = Class(
	function () {
		this.clear();
	},
	{
		clear: function() {
			this.map = {};
			this.count = 0;
		},

		containsKey: function(key) {
			return this.map.hasOwnProperty(key);
		},

		containsValue: function(value) {
			for (var key in this.map)
				if (this.map.hasOwnProperty(key) && this.map[key] == value)
					return true;
			return false;
		},

		get: function(key) {
			return this.map.hasOwnProperty(key) ? this.map[key] : null;
		},

		isEmpty: function() {
			return this.count == 0;
		},

		put: function(key, value) {
			if (this.map.hasOwnProperty(key)) {
				var oldValue = this.map[key];
				this.map[key] = value;
				return oldValue;
			} else {
				this.map[key] = value;
				this.count++;
				return null;
			}
		},

		remove: function(key) {
			if (this.map.hasOwnProperty(key)) {
				var value = this.map[key];
				delete this.map[key];
				this.count--;
				return value;
			} else
				return null;
		},
		
		size: function() {
			return this.count;
		}
	}
);

ns.Preferences = Class(
	function () {
	},
	null,
	{
		getNode: function(name) {
			if (window.localStorage)
				return new ns.WebStoragePreferences(window.localStorage, name);
			else
				return null;
		}
	}
);

ns.WebStoragePreferences = Class(
	function (storage, name) {
		this.storage = storage;
		this.root = 'jasq:' + name + '.';
	},
	{
		get: function(key, def) {
			var value = this.storage.getItem(this.root + key);
			return value == null ? def : value;
		},

		put: function(key, value) {
			this.storage.setItem(this.root + key, value);
		},

		remove: function(key) {
			this.storage.removeItem(this.root + key);
		}
	},
	null,
	ns.Preferences
);

ns.Queue = Class(
	function () {
		this.queue = [];
	},
	{
		add: function(elem) {
			this.queue.push(elem);
		},

		peek: function() {
			return this.queue.length == 0 ? null : this.queue[0];
		},

		remove: function() {
			return this.queue.length == 0 ? null : this.queue.shift();
		},

		size: function() {
			return this.queue.length;
		}
	}
);

ns.Random = Class(
	function (seed) {
		if (typeof seed == 'undefined')
			seed = new Date().getTime();
		this.setSeed(seed);
	},
	{
		_next: function(bits) {
			this.seed = ((this.seed * 16807) % 0xFFFFFFFF) | 0;
			return this.seed >>> (32 - bits);
		},

		nextBoolean: function() {
			return this._next(1) != 0;
		},

		nextDouble: function() {
			return (this._next(26) * 0x8000000 + this._next(27)) / 0x20000000000000;
		},

		nextInt: function(n) {
			if (typeof n == 'undefined')
				return this._next(32);
			else if (n <= 0)
				throw new RangeError('n must be positive');
			else {
				var bits, val;
				do {
					bits = this._next(31);
					val = (bits % n) | 0;
				} while (bits - val + (n-1) < 0);
				return val;
			}
		},

		setSeed: function(seed) {
			this.seed = ((seed ^ 16807) % 0xFFFFFFFF) | 0;
		}
	}
);

ns.Set = Class(
	function () {
		this.clear();
	},
	{
		add: function(elem) {
			if (this.set.hasOwnProperty(elem))
				return false;
			else {
				this.set[elem] = true;
				this.count++;
				return true;
			}
		},

		clear: function() {
			this.set = {};
			this.count = 0;
		},

		contains: function(elem) {
			return this.set.hasOwnProperty(elem);
		},

		isEmpty: function() {
			return this.count == 0;
		},

		remove: function(elem) {
			if (this.set.hasOwnProperty(elem)) {
				delete this.set[elem];
				this.count--;
				return true;
			} else
				return false;
		},

		size: function() {
			return this.count;
		},

		toArray: function() {
			var array = [];
			for (var elem in this.set)
				if (this.set.hasOwnProperty(elem))
					array.push(elem);
			return array;
		}
	}
);

ns.StringBuffer = Class(
	function (str) {
		this.str = str ? str : '';
	},
	{
		append: function(obj) {
			this.str += obj;
			return this;
		},

		insert: function(offset, obj) {
			this.str = this.str.substr(0, offset) + obj + this.str.substr(offset);
			return this;
		},

		toString: function() {
			return this.str;
		}
	}
);

ns.Timer = Class(
	function () {
		this.tasks = [];
		this.repeatTasks = [];
	},
	{
		schedule: function (task, delay, repeat) {
			if (repeat)
				this.repeatTasks.push(window.setInterval(task, delay));
			else
				this.tasks.push(window.setTimeout(task, delay));
		},

		cancel: function () {
			for (var i = 0; i < this.repeatTasks.length; i++)
				window.clearInterval(this.repeatTasks[i]);
			this.repeatTasks = [];

			for (var i = 0; i < this.tasks.length; i++)
				window.clearTimeout(this.tasks[i]);
			this.tasks = [];
		}
	}
);

window.jasq = ns;

})();

